using System;
using System.Web.UI.Design;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Web.UI.WebControls;
using System.Web.UI;
using System.IO;
namespace Interval.LoginTemplateControl
{
	/// <summary>
	/// Designer pro <see cref="LoginControl"/>
	/// </summary>
	public class LoginControlDesigner : TemplatedControlDesigner
	{
		
		#region private constants
		private const string LOGGEDIN_TEMPLATE_VERB = "Editovat ablonu LoggedInTemplate";
		private const string ANONYMOUS_TEMPLATE_VERB = "Editovat ablonu AnonymousTemplate";
		#endregion private constants

		#region private fields
		private TemplateEditingVerb[] m_tEVerbs;
		#endregion private fields

		#region Constructors
		public LoginControlDesigner() : base()
		{

		}
		#endregion Constructors
		
		#region Public properties
		/// <summary>
		/// Vlastnost uruje, kdy je mon v design mdu mnit velikost prvku
		/// </summary>
		public override bool AllowResize
		{
			get
			{
				LoginControl lc = (LoginControl) Component;
				return ((lc.AnonymousTemplate != null) || (lc.LoggedInTemplate != null) || (InTemplateMode));
			}
		}
		#endregion Public properties

		
		#region Public methods

		/// <summary>
		/// Inicializace komponenty
		/// </summary>
		/// <param name="component">Instance <see cref="LoginControl"/></param>
		public override void Initialize(IComponent component)
		{
			if (!(component is LoginControl))
				throw new ArgumentException("This designer is only for LoginControl", "component");
			
			base.Initialize (component);
		}

		/// <summary>
		/// Metoda vrac HTML, kterm je vykreslen prvek v design mdu
		/// </summary>
		/// <returns>HTML, kterm je vykreslen prvek v design mdu</returns>
		public override string GetDesignTimeHtml()
		{
			LoginControl lc = (LoginControl) Component;
			
			if ((lc.AnonymousTemplate == null) && (lc.LoggedInTemplate == null))
				return GetEmptyDesignTimeHtml();

			string retHTML = String.Empty;

			try
			{
				lc.DataBind();
				
				retHTML = base.GetDesignTimeHtml();

			}
			catch (Exception e)
			{
				retHTML = GetErrorDesignTimeHtml(e);
			}
			return retHTML;
		}

		/// <summary>
		/// Metoda vrt etzec s definic ablony
		/// </summary>
		/// <param name="editingFrame">Aktuln editovac rmec</param>
		/// <param name="templateName">Nzev ablony, jej obsah se m vrtit ve form textovho etzce</param>
		/// <param name="allowEditing">Vstupn argument, kter povoluje nebo zakazuje editaci ablony</param>
		/// <returns>etzec s definic ablony/returns>
		public override string GetTemplateContent(ITemplateEditingFrame editingFrame, string templateName, out bool allowEditing)
		{
			allowEditing = true;
			
			LoginControl lc = (LoginControl) Component;
			
			string retTemplate = String.Empty;
			
			if ((m_tEVerbs != null) && ((editingFrame.Verb.Text == LOGGEDIN_TEMPLATE_VERB) || (editingFrame.Verb.Text == ANONYMOUS_TEMPLATE_VERB)))
			{
				ITemplate template = (editingFrame.Verb.Text == LOGGEDIN_TEMPLATE_VERB ?  lc.LoggedInTemplate : lc.AnonymousTemplate);
				

				if (template != null)
					retTemplate = GetTextFromTemplate (template);
			}
			
			return retTemplate;
			
		}
		
		/// <summary>
		/// Metoda konvertuje textovou reprezentaci ablony na rozhran <see cref="ITemplate"/>, kter piad do vlastnosti <see cref="LoginControl.LoggedInTemplate"/> nebo kter piad do vlastnosti <see cref="LoginControl.AnonymousTemplate"/> 
		/// </summary>
		/// <param name="editingFrame">Aktuln editovac rmec</param>
		/// <param name="templateName">Nzev ablony, jej obsah se m vrtit ve form textovho etzce</param>
		/// <param name="allowEditing">Vstupn argument, kter povoluje nebo zakazuje editaci ablony</param>
		public override void SetTemplateContent(ITemplateEditingFrame editingFrame, string templateName, string templateContent)
		{
			
			LoginControl lc = (LoginControl) Component;
			
			
			if ((m_tEVerbs != null) && ((editingFrame.Verb.Text == LOGGEDIN_TEMPLATE_VERB) || (editingFrame.Verb.Text == ANONYMOUS_TEMPLATE_VERB)))
			{
				ITemplate newTemplate = null;
				

				if ((templateContent != null) && (templateContent != String.Empty))
					newTemplate = GetTemplateFromText(templateContent);

				if (editingFrame.Verb.Text == LOGGEDIN_TEMPLATE_VERB)
					lc.LoggedInTemplate = newTemplate;
				else
					lc.AnonymousTemplate = newTemplate;
			}
			
		}

		/// <summary>
		/// Pepsn metody odopovdn za vyvoln  udlosti <see cref="OnComponentChanged"/>
		/// </summary>
		/// <param name="sender">Zdroj udlosti</param>
		/// <param name="ce">Parametry udlosti</param>
		/// <remarks>
		/// Metoda uvoln existujc objekty <see cref="TemplateEditingVerb"/>, pokud se zmn font, barva poped nebo barva pozad v objektu <see cref="LoginControl"/>.
		///To zpsob, e jsou zmny ve zmnnch vlastnostech promtnuty do editovacho rmce ablon pi novm vytven objekt <see cref="TemplateEditingVerb"/>.
		///</remarks>
		public override void OnComponentChanged(object sender, ComponentChangedEventArgs ce)
		{
			base.OnComponentChanged (sender, ce);
			
			if (ce.Member != null)
			{
				if ((ce.Member.Name == "Font") || (ce.Member.Name == "ForeColor") || (ce.Member.Name == "BackColor"))
					clearVerbs();
			}
			
		}



		#endregion Public methods

		#region Protected methods
		
		/// <summary>
		/// Metoda vrac HTML, kterm je vykreslen prvek v design mdu v ppad, e jeho deklarace obsahuje chybu
		/// </summary>
		/// <returns>HTML, kterm je vykreslen prvek v design mdu v ppad, e jeho deklarace obsahuje chybu</returns>
		protected override string GetErrorDesignTimeHtml(Exception e)
		{
			return CreatePlaceHolderDesignTimeHtml("Prvek nen mon vykreslit!");
		}
		/// <summary>
		/// Metoda vrac HTML, kterm je vykreslen prvek v design mdu v ppad, e nebyly nadefinovny dn ablony
		/// </summary>
		/// <returns>HTML, kterm je vykreslen prvek v design mdu v ppad, e nebyly nadefinovny dn ablony</returns>
		protected override string GetEmptyDesignTimeHtml()
		{
			return CreatePlaceHolderDesignTimeHtml("Kliknte pravm tlatkem na prvek a z kontextovho menu vyberte ablonu, kterou chcete editovat.");
		}
		
		/// <summary>
		/// Metoda vytvo a vrt objekty <see cref="TemplateEditingVerb"/>, jejich nzvy budou v design mdu zobrazeny v kontextovm menu - po vybrn poloky ne uivatel editovat pslunou poloku
		/// </summary>
		/// <returns>objekty <see cref="TemplateEditingVerb"/>, jejich nzvy budou v design mdu zobrazeny v kontextovm menu - po vybrn poloky ne uivatel editovat pslunou poloku</returns>
		protected override TemplateEditingVerb[] GetCachedTemplateEditingVerbs()
		{
			if (m_tEVerbs == null)
			{
				m_tEVerbs = new TemplateEditingVerb[2];
				m_tEVerbs[0] = new TemplateEditingVerb(LOGGEDIN_TEMPLATE_VERB, 0, this);
				m_tEVerbs[1] = new TemplateEditingVerb(ANONYMOUS_TEMPLATE_VERB, 1, this);
			}
			
			return m_tEVerbs;

		}

		/// <summary>
		/// Vytvoen rmce, ve kterm bude probhat editace ablon
		/// </summary>
		/// <param name="verb">Objekt <see cref="TemplateEditingVerb"/>, kter uruje, jak ablona bude editovna</param>
		/// <returns>Rmec, ve kterm bude probhat editace ablon</returns>
		protected override ITemplateEditingFrame CreateTemplateEditingFrame(TemplateEditingVerb verb)
		{
			ITemplateEditingFrame retFrame = null;
			LoginControl lc = (LoginControl) Component;

			
			if ((m_tEVerbs != null) && ((verb.Text == LOGGEDIN_TEMPLATE_VERB) || (verb.Text == ANONYMOUS_TEMPLATE_VERB)))
			{
				ITemplateEditingService editingService = (ITemplateEditingService) GetService(typeof(ITemplateEditingService));
				
				if (editingService != null)
				{
					Style style = lc.ControlStyle;
					string[] templateName = {verb.Text};
					retFrame = editingService.CreateFrame(this, verb.Text, templateName, style, null);
				}

			}
			
			return retFrame;

		}


		#endregion Protected methods

		#region Private methods
		//Zruen existujcch objekt TemplateEditingVerb
		private void clearVerbs()
		{
			foreach (TemplateEditingVerb verb in m_tEVerbs)
				verb.Dispose();
			
			m_tEVerbs = null;
		}
		#endregion Private methods

	}
}
